# message types
CODE_NEW_EVALUATOR = 0x20
CODE_NEW_EVALUATOR_RESPONSE = 0x21
CODE_CLOSE_EVALUATOR = 0x22
CODE_EVALUATE = 0x23
CODE_EVALUATE_RESPONSE = 0x24
CODE_EVALUATE_LOG = 0x25
CODE_EVALUATE_READ = 0x26
CODE_EVALUATE_READ_RESPONSE = 0x27
CODE_EVALUATE_READ_MODULE = 0x28
CODE_EVALUATE_READ_MODULE_RESPONSE = 0x29
CODE_LIST_RESOURCES_REQUEST = 0x2A
CODE_LIST_RESOURCES_RESPONSE = 0x2B
CODE_LIST_MODULES_REQUEST = 0x2C
CODE_LIST_MODULES_RESPONSE = 0x2D
